"""
List all Xero SDK exceptions
"""

from .xerosdk import XeroSDK
from .exceptions import InvalidClientError
from .exceptions import InvalidGrant
from .exceptions import UnsupportedGrantType
from .exceptions import InternalServerError
from .exceptions import InvalidTokenError
from .exceptions import UnsuccessfulAuthentication

__all__ = [
    'XeroSDK',
    'InvalidClientError',
    'InvalidGrant',
    'UnsupportedGrantType',
    'InternalServerError',
    'InvalidTokenError',
    'UnsuccessfulAuthentication'
]



"""
Xero Items API
"""

from .api_base import ApiBase


class Items(ApiBase):
    """
    Class for Items API
    """

    GET_ITEMS = '/api.xro/2.0/Items'

    def get_all(self):
        """
        Get all Items

        Returns:
            List of all Items
        """

        return self._get_request(Items.GET_ITEMS)



"""
Xero SDK exceptions
"""


class XeroSDKError(Exception):
    """
    The base exception class for Xero SDK
    """

    def __init__(self, msg, response=None):
        super(XeroSDKError, self).__init__(msg)
        self.message = msg
        self.response = response

    def __str__(self):
        return repr(self.message)


class InvalidClientError(XeroSDKError):
    """Invalid client ID or client secret or refresh token, 400 error."""


class InvalidGrant(XeroSDKError):
    """Invalid refresh token, 400 error."""


class UnsupportedGrantType(XeroSDKError):
    """Invalid or non-existing grant type in request body, 400 error."""


class InvalidTokenError(XeroSDKError):
    """Invalid or non-existing access token, 401 error"""


class UnsuccessfulAuthentication(XeroSDKError):
    """Invalid xero tenant ID or xero-tenant-id header missing"""


class WrongParamsError(XeroSDKError):
    """Xero validation exception occurred, 400 error"""


class NoPrivilegeError(XeroSDKError):
    """The user has insufficient privilege, 403 error."""


class NotFoundItemError(XeroSDKError):
    """Not found the item from URL, 404 error."""


class InternalServerError(XeroSDKError):
    """Internal server error, 500 error"""



"""
List all Xero APIs
"""

from .invoices import Invoices
from .accounts import Accounts
from .contacts import Contacts
from .tracking_categories import TrackingCategories
from .payments import Payments
from .items import Items
from .tenants import Tenants
from .bank_transactions import BankTransactions
from .attachments import Attachments
from .organisations import Organisations

__all__ = [
    'Invoices',
    'Accounts',
    'Contacts',
    'TrackingCategories',
    'Payments',
    'Items',
    'Tenants',
    'BankTransactions',
    'Attachments',
    'Organisations'
]



"""
Xero TrackingCategories API
"""

from .api_base import ApiBase


class TrackingCategories(ApiBase):
    """
    Class for Tracking Categories API
    """

    GET_TRACKING_CATEGORIES = '/api.xro/2.0/trackingcategories'

    def get_all(self):
        """
        Get all tracking categories

        Returns:
            List of all tracking categories
        """
        return self._get_request(TrackingCategories.GET_TRACKING_CATEGORIES)



"""
Xero Invoices API
"""

from .api_base import ApiBase


class Invoices(ApiBase):
    """
    Class for Invoices API
    """

    GET_INVOICES = '/api.xro/2.0/invoices'
    GET_INVOICE_BY_ID = '/api.xro/2.0/invoices/{0}'
    POST_INVOICE = '/api.xro/2.0/invoices'

    def get_all(self):
        """
        Get all invoices

        Returns:
            List of all invoices
        """

        return self._get_request(Invoices.GET_INVOICES)

    def get_by_id(self, invoice_id):
        """
        Get invoice by invoice_id

        Parameters:
            invoice_id (str): Invoice ID

        Returns:
            Invoice dict
        """

        return self._get_request(Invoices.GET_INVOICE_BY_ID.format(invoice_id))

    def post(self, data):
        """
        Create new invoice

        Parameters:
            data (dict): Data to create invoice

        Returns:
             Response from API
        """

        return self._post_request(data, Invoices.POST_INVOICE)



"""
Xero Payments API
"""

from .api_base import ApiBase


class Payments(ApiBase):
    """
    Class for Payments API
    """

    GET_INVOICES = '/api.xro/2.0/payments'
    GET_INVOICE_BY_ID = '/api.xro/2.0/payments/{0}'
    POST_INVOICE = '/api.xro/2.0/payments'

    def get_all(self):
        """
        Get all payments

        Returns:
            List of all payments
        """

        return self._get_request(Payments.GET_INVOICES)

    def get_by_id(self, payment_id):
        """
        Get payment by payment_id

        Parameters:
            payment_id (str): Payment ID

        Returns:
            Payment dict
        """

        return self._get_request(Payments.GET_INVOICE_BY_ID.format(payment_id))

    def post(self, data):
        """
        Create new payment

        Parameters:
            data (dict): Data to create payment

        Returns:
             Response from API
        """

        return self._update_request(data, Payments.POST_INVOICE)



"""
API base class
"""

import json
import requests

from ..exceptions import *


class ApiBase:
    """
    Base class for all API classes
    """

    def __init__(self):
        self.__access_token = None
        self.__tenant_id = None
        self.__server_url = None

    def change_access_token(self, access_token):
        """
        Change old access token with a new one
        Parameters:
            access_token (str): New access token
        """

        self.__access_token = access_token

    def set_server_url(self, server_url):
        """
        Set server URL while creating a connection

        Parameters:
            server_url (str): Server URL for Xero API
        """

        self.__server_url = server_url

    def set_tenant_id(self, tenant_id):
        """
        Set tenant id while creating connection
        Parameters:
            tenant_id (str): Xero tenant ID
        """

        self.__tenant_id = tenant_id

    def _get_request(self, api_url):
        """
        HTTP get request to a given Xero API URL

        Parameters:
            api_url (str): URL of Xero API
        """

        api_headers = {
            'authorization': 'Bearer ' + self.__access_token,
            'xero-tenant-id': self.__tenant_id,
            'accept': 'application/json'
        }

        response = requests.get(
            self.__server_url+api_url,
            headers=api_headers
        )

        if response.status_code == 200:
            result = json.loads(response.text)
            return result

        if response.status_code == 403:
            raise UnsuccessfulAuthentication(
                'Invalid xero tenant ID or xero-tenant-id header missing'
            )

        if response.status_code == 500:
            raise InternalServerError(
                'Internal server error'
            )

        raise XeroSDKError(
            'Status code {0}'.format(response.status_code), response.text
        )

    def _update_request(self, data, api_url):
        """
        HTTP put method to send data to Xero API URL

        Parameters:
            data (dict): Data to be sent to Xero API
            api_url (str): URL of Xero API
        """

        api_headers = {
            'Authorization': 'Bearer ' + self.__access_token,
            'xero-tenant-id': self.__tenant_id,
            'accept': 'application/json'
        }

        response = requests.put(
            self.__server_url + api_url,
            headers=api_headers,
            json=data
        )

        if response.status_code == 200:
            result = json.loads(response.text)
            return result

        if response.status_code == 400:
            error_msg = json.loads(response.text)
            raise WrongParamsError(error_msg, response.status_code)

        if response.status_code == 401:
            error_msg = json.loads(response.text)
            raise InvalidTokenError('Invalid token, try to refresh it', error_msg)

        if response.status_code == 403:
            error_msg = json.loads(response.text)
            raise NoPrivilegeError('Forbidden, the user has insufficient privilege', error_msg)

        if response.status_code == 404:
            error_msg = json.loads(response.text)
            raise NotFoundItemError('Not found item with ID', error_msg)

        if response.status_code == 500:
            error_msg = json.loads(response.text)
            raise InternalServerError('Internal server error', error_msg)

        raise XeroSDKError(
            'Status code {0}'.format(response.status_code), response.text
        )

    def _post_request(self, data, api_url):
        """
        HTTP post method to send data to Xero API URL

        Parameters:
            data (dict): Data to be sent to Xero API
            api_url (str): URL of Xero API
        """

        api_headers = {
            'Authorization': 'Bearer ' + self.__access_token,
            'xero-tenant-id': self.__tenant_id,
            'accept': 'application/json'
        }

        response = requests.post(
            self.__server_url + api_url,
            headers=api_headers,
            json=data
        )

        if response.status_code == 200:
            result = json.loads(response.text)
            return result

        if response.status_code == 400:
            error_msg = json.loads(response.text)
            raise WrongParamsError(error_msg, response.status_code)

        if response.status_code == 401:
            error_msg = json.loads(response.text)
            raise InvalidTokenError('Invalid token, try to refresh it', error_msg)

        if response.status_code == 403:
            error_msg = json.loads(response.text)
            raise NoPrivilegeError('Forbidden, the user has insufficient privilege', error_msg)

        if response.status_code == 404:
            error_msg = json.loads(response.text)
            raise NotFoundItemError('Not found item with ID', error_msg)

        if response.status_code == 500:
            error_msg = json.loads(response.text)
            raise InternalServerError('Internal server error', error_msg)

        raise XeroSDKError(
            'Status code {0}'.format(response.status_code), response.text
        )

    def _get_tenant_ids(self):
        api_headers = {
            'authorization': 'Bearer ' + self.__access_token,
        }
        response = requests.get('https://api.xero.com/connections', headers=api_headers)

        if response.status_code == 200:
            return json.loads(response.text)

        if response.status_code == 400:
            error_msg = json.loads(response.text)
            raise WrongParamsError(error_msg, response.status_code)

        if response.status_code == 401:
            error_msg = json.loads(response.text)
            raise InvalidTokenError('Invalid token, try to refresh it', error_msg)

        if response.status_code == 403:
            error_msg = json.loads(response.text)
            raise NoPrivilegeError('Forbidden, the user has insufficient privilege', error_msg)

        if response.status_code == 404:
            error_msg = json.loads(response.text)
            raise NotFoundItemError('Not found item with ID', error_msg)

        if response.status_code == 500:
            error_msg = json.loads(response.text)
            raise InternalServerError('Internal server error', error_msg)

        raise XeroSDKError(
            'Status code {0}'.format(response.status_code), response.text
        )

    def _post_attachment(self, data, api_url):
        """Create a HTTP post request.

        Parameters:
            data: Data to be sent to Xero API
            api_url (str): Url of the Xero API.

        Returns:
            A response from the request (dict).
        """

        api_headers = {
            'Authorization': 'Bearer {}'.format(self.__access_token),
            'xero-tenant-id': self.__tenant_id,
            'Accept': 'application/json',
            'Content-Type': '*'
        }
        response = requests.post(
            '{0}{1}'.format(self.__server_url, api_url),
            headers=api_headers,
            data=data
        )

        if response.status_code == 200:
            result = json.loads(response.text)
            return result

        if response.status_code == 400:
            error_msg = response.text
            raise WrongParamsError(error_msg, response.status_code)

        if response.status_code == 401:
            error_msg = json.loads(response.text)
            raise InvalidTokenError('Invalid token, try to refresh it', error_msg)

        if response.status_code == 403:
            error_msg = response.text
            raise NoPrivilegeError('Forbidden, the user has insufficient privilege', error_msg)

        if response.status_code == 404:
            error_msg = json.loads(response.text)
            raise NotFoundItemError('Not found item with ID', error_msg)

        if response.status_code == 500:
            error_msg = json.loads(response.text)
            raise InternalServerError('Internal server error', error_msg)

        raise XeroSDKError(
            'Status code {0}'.format(response.status_code), response.text
        )



"""
Xero Tenants API
"""

from .api_base import ApiBase


class Tenants(ApiBase):
    """
    Class for Tenants
    """

    def get_all(self):
        """
        Get all Tenants

        Returns:
            List of all Tenants
        """

        return self._get_tenant_ids()



import json
from os import path
from unittest.mock import Mock

from xerosdk import XeroSDK


def get_mock_xero_dict(filename):
    basepath = path.dirname(__file__)
    filepath = path.join(basepath, filename)
    mock_xero_json = open(filepath, 'r').read()
    mock_xero_dict = json.loads(mock_xero_json)
    return mock_xero_dict


def get_mock_xero_from_file(filename):
    mock_xero_dict = get_mock_xero_dict(filename)
    mock_xero = Mock()
    mock_xero.Invoices.get_all.return_value = mock_xero_dict['invoices']
    mock_xero.Accounts.get_all.return_value = mock_xero_dict['accounts']
    mock_xero.Contacts.get_all.return_value = mock_xero_dict['contacts']
    mock_xero.TrackingCategories.get_all.return_value = mock_xero_dict['tracking_categories']
    return mock_xero


def get_mock_xero():
    return get_mock_xero_from_file('mock_xero.json')


def compare_dict_keys(dict1, dict2):
    """
    Compare dict1 keys with dict2 keys and see
    if dict1 has extra keys compared to dict2

    Parameters:
        dict1 (dict): response dict from API
        dict2 (dict): mock dict

    Returns:
        Set of keys
    """
    return dict1.keys() - dict2.keys()


def xero_connect():
    """
    Xero SDK connector

    Returns:
        XeroSDK class instance
    """
    file = open('test_credentials.json', 'r')
    xero_config = json.load(file)

    connection = XeroSDK(
        base_url=xero_config['base_url'],
        client_id=xero_config['client_id'],
        client_secret=xero_config['client_secret'],
        refresh_token=xero_config['refresh_token']
    )

    return connection



"""
Xero Accounts API
"""

from .api_base import ApiBase


class Accounts(ApiBase):
    """
    Class for Accounts API
    """

    GET_ACCOUNTS = '/api.xro/2.0/accounts'

    def get_all(self):
        """
        Get all accounts

        Returns:
            List of all accounts
        """

        return self._get_request(Accounts.GET_ACCOUNTS)



"""
Xero Attachments API
"""
from typing import BinaryIO

from .api_base import ApiBase


class Attachments(ApiBase):
    """Class for Attachments APIs."""

    GET_ATTACHMENT = '/api.xro/2.0/{0}/{1}/attachments'
    POST_ATTACHMENT = '/api.xro/2.0/{0}/{1}/attachments/{2}'

    def get_attachments(self, endpoint, guid):
        """Get the list of attachments

        Parameters:
            :param endpoint: The name of the parent endpoint (e.g. Receipts, Invoices)
            :param guid: The guid of the document that that attachment belongs to (e.g. ReceiptID or InvoiceID)

        Returns:
            List with dicts in Attachments schema.
        """
        return self._get_request(self.GET_ATTACHMENT.format(endpoint, guid))

    def post_attachment(self, endpoint, guid, filename, data: BinaryIO):
        """

        Parameters:
            :param data: Raw image content
            :param endpoint: The name of the parent endpoint (e.g. Receipts, Invoices)
            :param guid: The guid of the document that that attachment belongs to (e.g. ReceiptID or InvoiceID)
            :param filename: Name of the attachment

        Returns:
        """
        return self._post_attachment(data, self.POST_ATTACHMENT.format(endpoint, guid, filename))



from tests.common.utils import compare_dict_keys


def test_invoices(xero, mock_xero):
    """
    Test Xero Invoices object

    Parameters:
        xero (obj): Xero SDK instance
        mock_xero (obj): mock Xero SDK instance
    """

    invoices = xero.Invoices.get_all()['Invoices']
    mock_invoices = mock_xero.Invoices.get_all()

    assert compare_dict_keys(invoices[0], mock_invoices[0]) == set()
    assert compare_dict_keys(mock_invoices[0], invoices[0]) == set()


def test_accounts(xero, mock_xero):
    """
    Test Xero Accounts object

    Parameters:
        xero (obj): Xero SDK instance
        mock_xero (obj): mock Xero SDK instance
    """

    accounts = xero.Accounts.get_all()['Accounts']
    mock_accounts = mock_xero.Accounts.get_all()

    assert compare_dict_keys(accounts[0], mock_accounts[0]) == set()
    assert compare_dict_keys(mock_accounts[0], accounts[0]) == set()


def test_contacts(xero, mock_xero):
    """
    Test Xero Contacts object

    Parameters:
        xero (obj): Xero SDK instance
        mock_xero (obj): mock Xero SDK instance
    """

    contacts = xero.Contacts.get_all()['Contacts']
    mock_contacts = mock_xero.Contacts.get_all()

    assert compare_dict_keys(contacts[0], mock_contacts[0]) == set()
    assert compare_dict_keys(mock_contacts[0], contacts[0]) == set()


def test_tracking_categories(xero, mock_xero):
    """
    Test Xero TrackingCategories object

    Parameters:
        xero (obj): Xero SDK instance
        mock_xero (obj): mock Xero SDK instance
    """

    tracking_categories = xero.TrackingCategories.get_all()['TrackingCategories']
    mock_tracking_categories = mock_xero.TrackingCategories.get_all()

    assert compare_dict_keys(tracking_categories[0], mock_tracking_categories[0]) == set()
    assert compare_dict_keys(mock_tracking_categories[0], tracking_categories[0]) == set()



"""
Xero Organisations API
"""

from .api_base import ApiBase


class Organisations(ApiBase):
    """
    Class for Organisations API
    """

    GET_ORGANISATIONS = '/api.xro/2.0/Organisation'

    def get_all(self):
        """
        Get Organisations

        Returns:
            List of Organisations
        """

        return self._get_request(Organisations.GET_ORGANISATIONS)['Organisations']



"""
Xero Bank Transactions API
"""

from .api_base import ApiBase


class BankTransactions(ApiBase):
    """
    Class for Bank Transactions API
    """

    GET_BANK_TRANSACTIONS = '/api.xro/2.0/banktransactions'
    GET_BANK_TRANSACTION_BY_ID = '/api.xro/2.0/banktransactions/{0}'
    POST_BANK_TRANSACTION = '/api.xro/2.0/banktransactions'

    def get_all(self):
        """
        Get all Bank Transactions

        Returns:
            List of all Bank Transactions
        """

        return self._get_request(BankTransactions.GET_BANK_TRANSACTIONS)

    def get_by_id(self, bank_transaction_id):
        """
        Get bank transaction by bank_transaction_id

        Parameters:
            bank_transaction_id (str): Bank Transaction ID

        Returns:
            Bank Transaction dict
        """

        return self._get_request(
            BankTransactions.GET_BANK_TRANSACTION_BY_ID.format(bank_transaction_id)
        )

    def post(self, data):
        """
        Create new Bank Transaction

        Parameters:
            data (dict): Data to create bank transaction

        Returns:
             Response from API
        """

        return self._post_request(data, BankTransactions.POST_BANK_TRANSACTION)



import setuptools

with open('README.md', 'r') as f:
    long_description = f.read()


setuptools.setup(
    name='xerosdk',
    version='0.8.2',
    author='Ashwin T',
    author_email='ashwin.t@fyle.in',
    description='Python SDK to access Xero APIs',
    license='MIT',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords=['xero', 'api', 'python', 'sdk'],
    url='https://github.com/fylein/xero-sdk-py',
    packages=setuptools.find_packages(),
    install_requires=['requests==2.22.0'],
    classifiers=[
        'Topic :: Internet :: WWW/HTTP',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ]
)



import pytest

from tests.common.utils import get_mock_xero, xero_connect


@pytest.fixture
def mock_xero():
    return get_mock_xero()


@pytest.fixture(scope='module')
def xero():
    return xero_connect()



"""
Xero SDK connection base class
"""

import base64
import json

import requests

from .apis import *
from .exceptions import *


class XeroSDK:
    """
    Creates connection with Xero APIs using OAuth2 authentication

    Parameters:
        base_url (str): Base URL for Xero API
        client_id (str): Client ID for Xero API
        client_secret (str): Client Secret for Xero API
        refresh_token (str): Refresh token for Xero API
    """

    TOKEN_URL = 'https://identity.xero.com/connect/token'
    AUTHORIZE_URL = 'https://login.xero.com/identity/connect/authorize'

    def __init__(self, base_url, client_id, client_secret, refresh_token):
        # Store the input parameters
        self.__base_url = base_url
        self.__client_id = client_id
        self.__client_secret = client_secret
        self._refresh_token = refresh_token  # Fix: refresh token expiry
        self.__tenant_id = None

        # Create an object for each API
        self.invoices = Invoices()
        self.accounts = Accounts()
        self.contacts = Contacts()
        self.tracking_categories = TrackingCategories()
        self.payments = Payments()
        self.items = Items()
        self.tenants = Tenants()
        self.bank_transactions = BankTransactions()
        self.attachments = Attachments()
        self.organisations = Organisations()

        # Set the server url
        self.set_server_url()

        # Refresh access token
        self.refresh_access_token()

    def set_server_url(self):
        """
        Set server URL for all API objects
        """

        base_url = self.__base_url

        self.invoices.set_server_url(base_url)
        self.accounts.set_server_url(base_url)
        self.contacts.set_server_url(base_url)
        self.tracking_categories.set_server_url(base_url)
        self.items.set_server_url(base_url)
        self.tenants.set_server_url(base_url)
        self.payments.set_server_url(base_url)
        self.bank_transactions.set_server_url(base_url)
        self.attachments.set_server_url(base_url)
        self.organisations.set_server_url(base_url)

    def set_tenant_id(self, tenant_id):
        """
        Set tenant id for all API objects

        Parameters:
            tenant_id (str): Xero tenant ID
        """

        self.invoices.set_tenant_id(tenant_id)
        self.accounts.set_tenant_id(tenant_id)
        self.contacts.set_tenant_id(tenant_id)
        self.tracking_categories.set_tenant_id(tenant_id)
        self.items.set_tenant_id(tenant_id)
        self.payments.set_tenant_id(tenant_id)
        self.tenants.set_tenant_id(tenant_id)
        self.bank_transactions.set_tenant_id(tenant_id)
        self.attachments.set_tenant_id(tenant_id)
        self.organisations.set_tenant_id(tenant_id)

    def refresh_access_token(self):
        """
        Refresh access token for each API objects
        """

        access_token = self.__get_access_token()

        self.invoices.change_access_token(access_token)
        self.accounts.change_access_token(access_token)
        self.contacts.change_access_token(access_token)
        self.tracking_categories.change_access_token(access_token)
        self.items.change_access_token(access_token)
        self.payments.change_access_token(access_token)
        self.tenants.change_access_token(access_token)
        self.bank_transactions.change_access_token(access_token)
        self.attachments.change_access_token(access_token)
        self.organisations.change_access_token(access_token)

    def __get_access_token(self):
        """
        Get access token from Xero TOKEN_URL

        Returns:
            A new access token
        """

        api_headers = {
            'authorization': 'Basic ' + str(
                base64.b64encode(
                    (self.__client_id + ':' + self.__client_secret).encode('utf-8')
                ), 'utf-8'
            ),
        }
        api_data = {
            'grant_type': 'refresh_token',
            'refresh_token': self._refresh_token  # Fix: refresh token expiry
        }
        response = requests.post(XeroSDK.TOKEN_URL, headers=api_headers, data=api_data)

        if response.status_code == 200:
            token = json.loads(response.text)
            self._refresh_token = token['refresh_token']  # Fix: refresh token expiry
            return token['access_token']

        error_msg = json.loads(response.text)['error']
        if response.status_code == 400:
            if error_msg == 'invalid_client':
                raise InvalidClientError(
                    'Invalid client ID or client secret or refresh token'
                )

            if error_msg == 'invalid_grant':
                raise InvalidGrant(
                    'Invalid refresh token'
                )

            if error_msg == 'unsupported_grant_type':
                raise UnsupportedGrantType(
                    'Invalid or non-existing grant type in request body'
                )

            raise XeroSDKError(
                'Status code {0}'.format(response.status_code), response.text
            )

        if response.status_code == 500:
            raise InternalServerError(
                'Internal server error'
            )

        raise XeroSDKError(
            'Status code {0}'.format(response.status_code), response.text
        )

    @property
    def refresh_token(self):
        """
        Get the refresh_token
        """
        return self._refresh_token



"""
Xero Contacts API
"""

from .api_base import ApiBase


class Contacts(ApiBase):
    """
    Class for Contacts API
    """

    GET_CONTACTS = '/api.xro/2.0/contacts'
    POST_CONTACTS = '/api.xro/2.0/contacts'
    SEARCH_CONTACT = '/api.xro/2.0/Contacts?where=Name="{0}"'

    def get_all(self):
        """
        Get all contacts

        Returns:
            List of all contacts
        """

        return self._get_request(Contacts.GET_CONTACTS)

    def post(self, data):
        """
        create new contact

        Parameters:
        data (dict): Data to create contact

        Returns:
             Response from API
        """

        return self._post_request(data, Contacts.POST_CONTACTS)

    def search_contact_by_contact_name(self, contact_name: str):
        """
        Search contact by Contact Name
        :param contact_name: Xero Contact Name
        :return: Contact
        """

        response = self._get_request(Contacts.SEARCH_CONTACT.format(contact_name))

        return response['Contacts'][0] if response['Contacts'] else None
